/***************************************************************************/
/*                                                                         */
/* HDBENCH clone - a graphical benchmark software                          */
/* Copyright (C) 1999 Yuuki NINOMIYA, gm@smn.enjoy.ne.jp                   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License as published by    */
/* the Free Software Foundation; either version 2, or (at your option)     */
/* any later version.                                                      */
/*                                                                         */
/* This program is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/* GNU General Public License for more details.                            */
/*                                                                         */
/* You should have received a copy of the GNU General Public License       */
/* along with this program; if not, write to the                           */
/* Free Software Foundation, Inc., 59 Temple Place - Suite 330,            */
/* Boston, MA 02111-1307, USA.                                             */
/*                                                                         */
/***************************************************************************/

#define EXTERN extern

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include "intl.h"
#include "proto.h"
#include "variable.h"


gint get_meminfo(gint32 *totalmem,gint32 *freemem)
{
	FILE *fp;

#ifdef INFO_PROC		//procե륷ƥˤ
	gchar buf[255];
	gint32 buffers,cached;

	if((fp=fopen("/proc/meminfo","r"))==NULL){
		fprintf(stderr,_("Cannot open `/proc/meminfo'.\n"));
		return(FALSE);
	}
	fgets(buf,255,fp);
	fscanf(fp,"%*s %d %*d %d %*d %d %d",totalmem,freemem,&buffers,&cached);
	fclose(fp);

	*freemem+=(buffers+cached);

	return(TRUE);
#endif

#ifdef INFO_SYSCTL	//sysctlޥɤˤ
	if((fp=popen("/sbin/sysctl hw.physmem hw.usermem","r"))==NULL){
		fprintf(stderr,_("Cannot create a pipe.\n"));
		return(FALSE);
	}
	fscanf(fp,"%*s %d",totalmem);
	fscanf(fp,"%*s %d",freemem);
	if(feof(fp)){
		fprintf(stderr,_("Cannot execute `/sbin/sysctl'.\n"));
		return(FALSE);
	}
	pclose(fp);

	return(TRUE);
#endif	

	return(FALSE);
}


void get_cpuinfo(gchar *name,gchar *vendor,gchar *family,gchar *model,gchar *stepping)
{
	FILE *fp;
	gchar model_name[64],clock[16];

#ifdef INFO_SYSCTL	//sysctlޥɤˤ
	gint processor;
	gchar clock_temp[16];
	gint i;
#endif

#ifdef INFO_PROC		//procե륷ƥˤ
	gchar item[20],val[255],buf[255],processor[4];
	gchar *ptr;

	strcpy(vendor,"unknown");
	strcpy(family,"unknown");
	strcpy(model,"unknown");
	strcpy(stepping,"unknown");
	strcpy(model_name,"unknown");
	strcpy(clock,"unknown");

	if((fp=fopen("/proc/cpuinfo","r"))==NULL){
		g_error(_("Cannot open `/proc/cpuinfo'.\n"));
	}

	for(;;){
		fgets(buf,255,fp);
		if(feof(fp)){
			break;
		}
		if(buf[0]=='\n'){
			continue;
		}

		ptr=strchr(buf,'\t');
		*ptr='\0';
		strcpy(item,buf);

		strcpy(val,strchr(ptr+1,':')+2);
		*strchr(val,'\n')='\0';

		if(strcmp(item,"model")==0){
			strcpy(model,val);
		}
		if(strcmp(item,"model name")==0){
			strcpy(model_name,val);
		}
		if(strcmp(item,"cpu MHz")==0){
			strcpy(clock,val);
		}
		if(strcmp(item,"vendor_id")==0){
			strcpy(vendor,val);
		}
		if(strcmp(item,"cpu family")==0){
			strcpy(family,val);
		}
		if(strcmp(item,"stepping")==0){
			strcpy(stepping,val);
		}
		if(strcmp(item,"processor")==0){
			strcpy(processor,val);
		}
	}
	fclose(fp);

	if(strcmp(model_name,"unknown")==0){
		strcpy(model_name,model);
		strcpy(model,"unknown");
	}

	if(processor[0]=='0'){
		sprintf(name,"%s [%s MHz]",model_name,clock);
	}else{
		sprintf(name,"%s [%s MHz] %d processors",model_name,clock,atoi(processor)+1);
	}

	return;
#endif

#ifdef INFO_SYSCTL	//sysctlޥɤˤ
	if((fp=popen("/sbin/sysctl hw.model machdep.tsc_freq hw.ncpu","r"))==NULL){
		g_error(_("Cannot create a pipe.\n"));
	}
	fscanf(fp,"%*s ");
	if(feof(fp)){
		g_error(_("Cannot execute `/sbin/sysctl'.\n"));
	}
	fgets(model_name,255,fp);
	*strchr(model_name,'\n')='\0';
	fscanf(fp,"%*s %s",clock_temp);
	fscanf(fp,"%*s %d",&processor);
	pclose(fp);

	for(i=strlen(clock_temp);i>strlen(clock_temp)-6;i--){
		clock[i]=clock_temp[i-1];
	}
	clock[i]='.';
	for(i=strlen(clock_temp)-7;i>=0;i--){
		clock[i]=clock_temp[i];
	}
	clock[strlen(clock_temp)+1]='\0';

	if(processor==1){
		sprintf(name,"%s [%s MHz]",model_name,clock);
	}else{
		sprintf(name,"%s [%s MHz] %d processors",model_name,clock,processor);
	}

	strcpy(vendor,"unknown");
	strcpy(family,"unknown");
	strcpy(model,"unknown");
	strcpy(stepping,"unknown");

	return;
#endif	
	g_error(_("Cannot get cpu infomation.\n"));
}


void get_osinfo(gchar *name)
{
	FILE *fp;

	if((fp=popen("uname -s -r","r"))==NULL){
		g_error(_("Cannot create a pipe.\n"));
	}
	fgets(name,255,fp);
	if(feof(fp)){
		g_error(_("Cannot execute `uname'.\n"));
	}
	*strchr(name,'\n')='\0';
	pclose(fp);
}


gchar *get_display_name(void)
{
	FILE *fp;
	gchar *temp;

	fp=popen("grep BoardName /etc/X11/XF86Config","r");
	if(fp==NULL){
		g_error(_("Cannot create a pipe.\n"));
	}
	temp=parse_display_name(fp);

	if(strcmp(temp,"unknown")==0){
		fp=popen("grep BoardName /etc/XF86Config","r");
		if(fp==NULL){
			g_error(_("Cannot create a pipe.\n"));
		}
		temp=parse_display_name(fp);

		if(strcmp(temp,"unknown")==0){
			g_warning(_("Cannot execute `grep' or open `XF86Config'.\n"));
		}
	}

	return(temp);
}


gchar *parse_display_name(FILE *fp)
{
	gchar temp[255];
	gint i,j;

	while(fgets(temp,255,fp)!=NULL){
		for(i=0;i<255;i++){
			if(temp[i]=='#' || temp[i]=='\n'){
				break;
			}
			if(temp[i]=='\"'){
				for(j=i+1;j<255;j++){
					if(temp[j]=='\n'){	//Ĥ'\"'ʤ
						pclose(fp);
						return(g_strdup("unknown"));
					}
					if(temp[j]=='\"'){
						temp[j]='\0';
						pclose(fp);
						return(g_strdup(temp+i+1));
					}
				}
			}
		}
	}
	pclose(fp);
	return(g_strdup("unknown"));
}


guint32 lpow(gint x,gint y)
{
	gint i;
	gint32 ret;

	if(x==2 && y==32){
		return(0x00ffffff);
	}
	ret=x;
	for(i=1;i<y;i++){
		ret*=x;
	}
	return(ret);
}


gchar *get_drive_data(void)
{
	gint i;
	gchar *temp;
	gchar *ret;
	FILE *fp;
	gchar cmd[40];
	gchar buf[255];

	gchar *grep[]={
		"^ide:",
		"^scsi0 :",
		"^scsi1 :",
		"^scsi2 :",
		"^scsi3 :",
		"^hda:",
		"^hdb:",
		"^hdc:",
		"^hdd:",
		"^da0:",
		"^da1:",
		"^da2:",
		"^da3:",
		"^SCSI device sda:",
		"^SCSI device sdb:",
		"^SCSI device sdc:",
		"^SCSI device sdd:",
		"^sr0:",
		"^sr1:",
		"^sr2:",
		"^sr3:",	/* 21 */
	};


	ret=g_strdup("");

	for(i=0;i<21;i++){
		sprintf(cmd,"dmesg | grep '%s' | tac",grep[i]);
		fp=popen(cmd,"r");
		if(fp==NULL){
			g_error(_("Cannot create a pipe.\n"));
		}
		for(;;){
			if(fgets(buf,255,fp)==NULL){
				break;
			}
			if(strstr(buf,"media changed")==NULL){
				temp=g_strconcat(ret,buf,NULL);
				g_free(ret);
				ret=temp;
				break;
			}
		}
		pclose(fp);
	}

	fp=popen("df","r");
	if(fp==NULL){
		g_error(_("Cannot create a pipe.\n"));
	}
	fgets(buf,255,fp);
	if(feof(fp)){
		g_warning(_("Cannot execute `df'.\n"));
		return(ret);
	}

	if(strncmp(buf,"Filesystem",10)==0){	//VineϤĤޤܸˤʤäƤΤͤ
		temp=g_strconcat(ret,"\n",buf,NULL);
	}else{
		temp=g_strconcat(ret,"\nFilesystem         1024-blocks  Used Available Capacity Mounted on\n",NULL);
	}
	g_free(ret);
	ret=temp;

	while(fgets(buf,255,fp)!=NULL){
		temp=g_strconcat(ret,buf,NULL);
		g_free(ret);
		ret=temp;
	}
	return(ret);
}



